/*  libfreenet
 *  Copyright 2001 Steven Hazel <sah@thalassocracy.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>

#include "status.h"
#include "protocol.h"
#include "message.h"


int freenet_message_create(freenet_message *message)
{
  message->current_line = 0;
  message->numfields = 0;
  message->type = FN_UNKNOWN_MSG;
  message->uniqueid = 0;
  message->fields = malloc(sizeof(freenet_field) * FN_MAX_FIELDS);
  if (message->fields == NULL) {
    return FNS_MALLOC_FAILED;
  }

  return FNS_SUCCESS;
}


int freenet_message_get_field(freenet_message *message, const char *name,
                              char *value)
{
  int namelen, i;

  namelen = strlen(name);

  for (i = 0; i < message->numfields; i++) {
    if (strncmp(message->fields[i].name, name, namelen) == 0) {
      strncpy(value, message->fields[i].value,
              strlen(message->fields[i].value));
      value[strlen(message->fields[i].value)] = 0;
      return FNS_SUCCESS;
    }
  }

  return FNS_NO_SUCH_FIELD;
}


int freenet_message_set_field(freenet_message *message, const char *name,
                              const char *value)
{
  int retval, i;

  if (strlen(value) > FN_MAX_FIELD_LEN) {
    return FNS_FIELD_TOO_LONG;
  }

  i = message->numfields;

  /* DO NOT ATTEMPT TO RETURN BEFORE SETTING i ABOVE */

  message->fields[i].name = malloc(strlen(name));
  if (message->fields[i].name == NULL) {
    retval = FNS_MALLOC_FAILED;
    goto end;
  }

  strncpy(message->fields[i].name, name, strlen(name));
  message->fields[i].name[strlen(name)] = 0;

  message->fields[i].value = malloc(strlen(value));
  if (message->fields[i].value == NULL) {
    retval = FNS_MALLOC_FAILED;
    goto end;
  }

  strncpy(message->fields[i].value, value, strlen(value));
  message->fields[i].value[strlen(value)] = 0;

  message->numfields++;
  retval = FNS_SUCCESS;

 end:
  if (retval != FNS_SUCCESS) {
    free(message->fields[i].name);
    free(message->fields[i].value);
  }
  return retval;
}


int freenet_message_get_next_line(freenet_message *message,
                                  const char *separator, char *line)
{
  int i;
  int vallen, namelen, seplen;

  i = message->current_line;

  if (i < message->numfields) {

    namelen = strlen(message->fields[i].name);
    vallen = strlen(message->fields[i].value);
    seplen = strlen(separator);

    if ((namelen + vallen + seplen
         + 1) /* strlen("\n") */
        >= FN_MAX_FIELD_LEN) {
      return FNS_LINE_TOO_LONG;
    }
    strncpy(line, message->fields[i].name, namelen);
    strncpy(&(line[namelen]), separator, seplen);
    strncpy(&(line[namelen+seplen]), message->fields[i].value, vallen);
    line[namelen+seplen+vallen] = 0;

    message->current_line++;

    return FNS_SUCCESS;
  } else {
    return FNS_MESSAGE_FINISHED;
  }
}


int freenet_message_destroy(freenet_message *message)
{
  int i;

  for (i = 0; i < message->numfields; i++) {
    free(message->fields[i].name);
    free(message->fields[i].value);
  }
  free(message->fields);

  message->current_line = 0;
  message->numfields = 0;
  message->type = FN_UNKNOWN_MSG;
  message->uniqueid = 0;

  return FNS_SUCCESS;
}
